@@ -1,10 +1,10 @@
-# -*- mode: cperl; coding: latin-2 -*-
+# -*- perl -*-
 
 #
-# $Id: Kwalify.pm,v 1.4 2007/03/04 10:50:06 eserte Exp $
+# $Id: Kwalify.pm,v 1.2 2006/11/23 21:19:59 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 2006,2007 Slaven Rezic. All rights reserved.
+# Copyright (C) 2006 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -15,10 +15,8 @@
 package Schema::Kwalify;
 
 use strict;
-use warnings;
-
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 use Kwalify qw();
 
@@ -28,7 +26,7 @@ sub new {
 
 sub validate {
     my($self, $schema, $data) = @_;
-    Kwalify::validate($schema, $data);
+    Kwalify::kwalify($schema, $data);
 }
 
 1;
@@ -39,7 +37,7 @@ __END__
 
 =head1 NAME
 
-Schema::Kwalify - Kwalify implementation in the Schema:: namespace
+Schema::Kwalify - 
 
 =head1 SYNOPSIS
 
